
#include <iuptreeutil.h>
#include <cstdio>
#include <cstring>
#include <cassert>
#include <algorithm>

static void copyString(char** dest, char* src)
{
  if(*dest)
    delete [] *dest;

  *dest = new char[strlen(src) + 1];
  strcpy(*dest, src);
}


/*****************************************************************************\
*
* IupLeafWrapper
*
\*****************************************************************************/

//#define DEBUG_TREE_UTIL

IupLeafWrapper::IupLeafWrapper(char* name, void* data)
{
  _data = data;
  _name = 0;
  _parent = 0;
  copyString(&_name, name);
}


int IupLeafWrapper::id(void)
{
  return IupTreeGetId(_tree, this);
}


void IupLeafWrapper::setName(char* name)
{
  copyString(&_name, name);
  char str[1024];
  sprintf(str, "NAME%d", id());
  IupSetAttribute(_tree, str, name);
}


int IupLeafWrapper::selection(int status)
{
#ifdef DEBUG_TREE_UTIL
  printf("IupLeafWrapper::selection : %d (%d)\n", id(), status);
#endif
  return IUP_DEFAULT;
}


int IupLeafWrapper::execute(char* name)
{
#ifdef DEBUG_TREE_UTIL
  printf("IupLeafWrapper::execute : %d (%s)\n", id(), name);
#endif
  return IUP_DEFAULT;
}


int IupLeafWrapper::rename(char* name)
{
#ifdef DEBUG_TREE_UTIL
  printf("IupLeafWrapper::rename : %d (%s)\n", id(), name);
#endif
  return IUP_DEFAULT;
}


int IupLeafWrapper::rightClick(void)
{
#ifdef DEBUG_TREE_UTIL
  printf("IupLeafWrapper::rightClick : %d\n", id());
#endif
  return IUP_DEFAULT;
}


/*****************************************************************************\
*
* IupTreeWrapper
*
\*****************************************************************************/


static bool isLeaf(Ihandle* tree, int id)
{
  char str[512];
  sprintf(str, "KIND%d", id);

  if(strcmp(IUP_LEAF, IupGetAttribute(tree, str)) == 0)
    return true;

  return false;
}


/*
* Funcoes estaticas de callbacks
*
*/

static int IupTreeWrapper_RightClick(Ihandle* self, int id)
{
  if(isLeaf(self, id))
  {
    IupLeafWrapper* temp = (IupLeafWrapper*) IupTreeGetUserId(self, id);

    if(temp)
      return temp->rightClick();
  }
  else
  {
    IupTreeWrapper* temp = (IupTreeWrapper*) IupTreeGetUserId(self, id);

    if(temp)
      return temp->rightClick();
  }

  return IUP_DEFAULT;
}


static int IupTreeWrapper_Selection(Ihandle* self, int id, int status)
{
  if(isLeaf(self, id))
  {
    IupLeafWrapper* temp = (IupLeafWrapper*) IupTreeGetUserId(self, id);

    if(temp)
      return temp->selection(status);
  }
  else
  {
    IupTreeWrapper* temp = (IupTreeWrapper*) IupTreeGetUserId(self, id);

    if(temp)
      return temp->selection(status);
  }

  return IUP_DEFAULT;
}


static int IupTreeWrapper_BranchOpen(Ihandle* self, int id)
{
  assert(!isLeaf(self, id));

  IupTreeWrapper* temp = (IupTreeWrapper*) IupTreeGetUserId(self, id);

  if(temp)
    return temp->branchOpen();

  return IUP_DEFAULT;
}


static int IupTreeWrapper_BranchClose(Ihandle* self, int id)
{
  assert(!isLeaf(self, id));

  IupTreeWrapper* temp = (IupTreeWrapper*) IupTreeGetUserId(self, id);

  if(temp)
    return temp->branchClose();

  return IUP_DEFAULT;
}


static int IupTreeWrapper_ExecuteLeaf(Ihandle* self, int id, char* name)
{
  assert(isLeaf(self, id));

  IupLeafWrapper* temp = (IupLeafWrapper*) IupTreeGetUserId(self, id);

  printf("DEBUG : %s\n", name);

  if(temp)
    return temp->execute(name);

  return IUP_DEFAULT;
}


static int IupTreeWrapper_Rename(Ihandle* self, int id, char* name)
{
  if(isLeaf(self, id))
  {
    IupLeafWrapper* temp = (IupLeafWrapper*) IupTreeGetUserId(self, id);

    if(temp)
      return temp->rename(name);
  }
  else
  {
    IupTreeWrapper* temp = (IupTreeWrapper*) IupTreeGetUserId(self, id);

    if(temp)
      return temp->rename(name);
  }

  return IUP_DEFAULT;
}


/*
* Implementacao da classe IupTreeWrapper
*
*/

IupTreeWrapper::IupTreeWrapper(Ihandle* tree, char* rootName, void* data)
{
  _tree = tree;
  _data = data;
  _name = 0;
  _parent = 0;

  copyString(&_name, rootName);

  IupTreeSetUserId(_tree, 0, this); 
  IupStoreAttribute(_tree, "NAME0", rootName);

  IupSetAttribute(_tree, "WRAPPER", (char*) this);

  IupSetAttribute(_tree, IUP_RIGHTCLICK_CB, "IupTreeWrapper_RightClick");
  IupSetFunction("IupTreeWrapper_RightClick", (Icallback) IupTreeWrapper_RightClick);

  IupSetAttribute(_tree, IUP_SELECTION_CB, "IupTreeWrapper_Selection");
  IupSetFunction("IupTreeWrapper_Selection", (Icallback) IupTreeWrapper_Selection);

  IupSetAttribute(_tree, IUP_BRANCHOPEN_CB, "IupTreeWrapper_BranchOpen");
  IupSetFunction("IupTreeWrapper_BranchOpen", (Icallback) IupTreeWrapper_BranchOpen);

  IupSetAttribute(_tree, IUP_BRANCHCLOSE_CB, "IupTreeWrapper_BranchClose");
  IupSetFunction("IupTreeWrapper_BranchClose", (Icallback) IupTreeWrapper_BranchClose);

  IupSetAttribute(_tree, IUP_EXECUTELEAF_CB, "IupTreeWrapper_ExecuteLeaf");
  IupSetFunction("IupTreeWrapper_ExecuteLeaf", (Icallback) IupTreeWrapper_ExecuteLeaf);

  IupSetAttribute(_tree, IUP_RENAMENODE_CB, "IupTreeWrapper_Rename");
  IupSetFunction("IupTreeWrapper_Rename", (Icallback) IupTreeWrapper_Rename);
}


IupTreeWrapper::IupTreeWrapper(char* name, void* data)
{
  _tree = 0;
  _name = 0;
  _parent = 0;
  copyString(&_name, name);
  _data = data;
}


void IupTreeWrapper::addLeafFront(IupLeafWrapper* leaf)
{
  int pos = leafFrontPosition();

  char str[1024];
  sprintf(str, "ADDLEAF%d", pos - 1);   
  IupStoreAttribute(_tree, str, leaf->getName());

  IupTreeSetUserId(_tree, pos, leaf);

  leaf->setTree(_tree);
  leaf->_parent = this;

  _leaves.push_front(leaf);

  int depth = getDepth(this);
  setDepth(leaf, depth + 1);
}


void IupTreeWrapper::addLeafBack(IupLeafWrapper* leaf)
{
  int pos = leafBackPosition();

  char str[1024];
  sprintf(str, "ADDLEAF%d", pos - 1);   
  IupStoreAttribute(_tree, str, leaf->getName());

  leaf->setTree(_tree);
  leaf->_parent = this;

  IupTreeSetUserId(_tree, pos, leaf);

  _leaves.push_back(leaf);

  int depth = getDepth(this);
  setDepth(leaf, depth + 1);
}


IupLeafWrapper* IupTreeWrapper::findLeaf(void* data, bool searchSubTrees)
{
  LeafList::iterator i;
  for(i = _leaves.begin(); i != _leaves.end(); i++)
  {
    if((*i)->getData() == data)
      return *i;
  }

  if(searchSubTrees)
  {
    TreeList::iterator j;
    for(j = _subTrees.begin(); j != _subTrees.end(); j++)
    {
      IupLeafWrapper* temp = (*j)->findLeaf(data, searchSubTrees);

      if(temp != 0)
        return temp;
    }
  }

  return 0;
}


void IupTreeWrapper::addSubTreeFront(IupTreeWrapper* subTree)
{
  int pos = subTreeFrontPosition();

  char str[1024];
  sprintf(str, "ADDBRANCH%d", pos - 1);   
  IupStoreAttribute(_tree, str, subTree->getName());

  subTree->setTree(_tree);
  subTree->_parent = this;

  IupTreeSetUserId(_tree, pos, subTree);

  _subTrees.push_front(subTree);

  int depth = getDepth(this);
  setDepth(subTree, depth + 1);
}


void IupTreeWrapper::addSubTreeBack(IupTreeWrapper* subTree)
{
  int pos = subTreeBackPosition();

  char str[1024];
  sprintf(str, "ADDBRANCH%d", pos - 1);   
  IupStoreAttribute(_tree, str, subTree->getName());

  subTree->setTree(_tree);
  subTree->_parent = this;

  IupTreeSetUserId(_tree, pos, subTree);

  _subTrees.push_back(subTree);

  int depth = getDepth(this);
  setDepth(subTree, depth + 1);
}


int IupTreeWrapper::subTreeFrontPosition(void)
{
  return IupTreeGetId(_tree, this) + 1;
}


int IupTreeWrapper::subTreeBackPosition(void)
{
  // adding after the root
  if(_subTrees.empty())
    return IupTreeGetId(_tree, this) + 1;

  // looking for a position, by traversing the subtrees
  IupTreeWrapper* p = _subTrees.back();
  IupTreeWrapper* q = 0;

  int maxPos;

  while(p)
  {
    q = p;

    if(p->_leaves.empty() == false)
      return IupTreeGetId(_tree, p->_leaves.back()) + 1;

    if(p->_subTrees.empty())
      p = 0;
    else
      p = p->_subTrees.back();
  }

  assert(q != 0);
  assert(q->_leaves.empty() == true);

  return IupTreeGetId(_tree, q) + 1;

  /*
  if(_subTrees.empty())
  return IupTreeGetId(_tree, this) + 1;

  IupTreeWrapper* p = _subTrees.back();
  IupTreeWrapper* q = 0;

  while(p)
  {
  q = p;

  if(p->_subTrees.empty())
  p = 0;
  else
  p = p->_subTrees.back();
  }

  assert(q != 0);

  if(q->_leaves.empty() == false)
  return IupTreeGetId(_tree, q->_leaves.back()) + 1;

  return IupTreeGetId(_tree, q) + 1;
  */
}


int IupTreeWrapper::leafFrontPosition(void)
{
  return subTreeBackPosition();
}


int IupTreeWrapper::leafBackPosition(void)
{
  if(_leaves.empty())
    return leafFrontPosition();

  return IupTreeGetId(_tree, _leaves.back()) + 1;
}


void IupTreeWrapper::redraw(void)
{
  IupSetAttribute(_tree, IUP_REDRAW, 0);
}


int IupTreeWrapper::getDepth(void* userId)
{
  int id = IupTreeGetId(_tree, userId);
  char str[1024];
  sprintf(str, "DEPTH%d", id);
  return IupGetInt(_tree, str);
}


void IupTreeWrapper::setDepth(void* userId, int depth)
{
  int id = IupTreeGetId(_tree, userId);
  char str[1024];
  sprintf(str, "DEPTH%d", id);
  IupSetfAttribute(_tree, str, "%d", depth);
}


bool IupTreeWrapper::removeLeaf(IupLeafWrapper* leaf)
{
  LeafList::iterator i = std::find(_leaves.begin(), _leaves.end(), leaf);

  if(i == _leaves.end())
    return false;

  int leafId = leaf->id();
  IupTreeSetUserId(_tree, leafId, 0);
  char str[1024];
  sprintf(str, "DELNODE%d", leafId);
  IupStoreAttribute(_tree, str, IUP_SELECTED);

  _leaves.erase(i);
  leaf->_parent = 0;

  return true;
}


bool IupTreeWrapper::removeLeaves(bool includeSubTrees, bool deleteLeaves)
{
  if(includeSubTrees)
  {
    TreeList::iterator i;
    for(i = _subTrees.begin(); i != _subTrees.end(); i++)
      (*i)->removeLeaves(includeSubTrees, deleteLeaves);
  }

  LeafList::iterator j;
  for(j = _leaves.begin(); j != _leaves.end(); j++)
  {
    IupLeafWrapper* leaf = *j;

    int leafId = leaf->id();
    IupTreeSetUserId(_tree, leafId, 0);
    char str[1024];
    sprintf(str, "DELNODE%d", leafId);
    IupStoreAttribute(_tree, str, IUP_SELECTED);

    leaf->_parent = 0;

    if(deleteLeaves)
      delete leaf;
  }

  _leaves.clear();

  return true;
}


bool IupTreeWrapper::removeSubTrees(bool deleteSubTrees, bool deleteLeaves)
{
  removeLeaves(true, deleteLeaves);

  TreeList::iterator i;
  for(i = _subTrees.begin(); i != _subTrees.end(); i++)
  {
    (*i)->removeSubTrees(deleteSubTrees, deleteLeaves);

    int treeId = (*i)->id();
    IupTreeSetUserId(_tree, treeId, 0);
    char str[1024];
    sprintf(str, "DELNODE%d", treeId);
    IupStoreAttribute(_tree, str, IUP_SELECTED);

    (*i)->_parent = 0;

    if(deleteSubTrees)
      delete (*i);
  }

  _subTrees.clear();

  return true;  
}


int IupTreeWrapper::tree(void)
{
  return IupTreeGetId(_tree, this);
}


int IupTreeWrapper::id(void)
{
  return IupTreeGetId(_tree, this);
}


int IupTreeWrapper::rightClick(void)
{
#ifdef DEBUG_TREE_UTIL
  printf("IupTreeWrapper::rightClick -> %d\n", id());
#endif
  return IUP_DEFAULT;
}


int IupTreeWrapper::selection(int status)
{
#ifdef DEBUG_TREE_UTIL
  printf("IupTreeWrapper::selection %d (%d)\n", id(), status);
#endif
  return IUP_DEFAULT;
}


int IupTreeWrapper::branchOpen(void)
{
#ifdef DEBUG_TREE_UTIL
  printf("IupTreeWrapper::branchOpen -> %d\n", id());
#endif
  return IUP_DEFAULT;
}


int IupTreeWrapper::branchClose(void)
{
#ifdef DEBUG_TREE_UTIL
  printf("IupTreeWrapper::branchClose -> %d\n", id());
#endif
  return IUP_DEFAULT;
}


int IupTreeWrapper::rename(char* name)
{
#ifdef DEBUG_TREE_UTIL
  printf("IupTreeWrapper::rename -> %d -> %s\n", id(), name);
#endif
  return IUP_DEFAULT;
}


void IupTreeWrapper::setName(char* name)
{
  copyString(&_name, name);
  char str[1024];
  sprintf(str, "NAME%d", id());
  IupSetAttribute(_tree, str, name);
}

