
#ifndef _IUP_TREE_UTIL_H_
#define _IUP_TREE_UTIL_H_


#include <iup.h>
#include <iuptree.h>
#include <list>

class IupLeafWrapper;
class IupTreeWrapper;


class IupLeafWrapper
{
public:
  IupLeafWrapper(char* nome, void* data = 0);

  IupTreeWrapper* getParent(void) { return _parent; }

  void setData(void* data) { _data = data; }

  void* getData() { return _data; }

  int id(void);

  char* getName(void) { return _name; }
  void setName(char* nome);

  // callbacks
  virtual int selection(int status);

  virtual int execute(char* name);

  virtual int rename(char* name);

  virtual int rightClick(void);

protected:

  friend class IupTreeWrapper;

  IupTreeWrapper* _parent;

  void setTree(Ihandle* tree) { _tree = tree; }

  Ihandle* _tree;
  char* _name;
  void* _data;
};


class IupTreeWrapper
{
public:
  IupTreeWrapper(Ihandle* tree, char* rootName, void* data = 0);
  IupTreeWrapper(char* name, void* data = 0);

  void setData(void* data) { _data = data; }

  void* getData() { return _data; }

  IupTreeWrapper* getParent() { return _parent; }

  // Addition of leaves and branches
  void addLeafFront(IupLeafWrapper* leaf);
  void addLeafBack(IupLeafWrapper* leaf);

  void addSubTreeFront(IupTreeWrapper* subTree);
  void addSubTreeBack(IupTreeWrapper* subTree);

  //
  IupLeafWrapper* findLeaf(void* data, bool searchSubTrees = true);

  // removal of leaves and branches
  bool removeLeaf(IupLeafWrapper* leaf);

  bool removeLeaves(bool includeSubTrees = true, bool deleteLeaves = false);

  bool removeSubTrees(bool deleteSubTrees = false, bool deleteLeaves = false);

  int id(void);

  void redraw(void);

  // callbacks

  virtual int selection(int status);

  virtual int branchOpen(void);

  virtual int branchClose(void);

  virtual int rename(char* name);

  virtual int rightClick(void);

  void setName(char* name);
  char* getName(void) { return _name; }

protected:

  typedef std::list<IupTreeWrapper*> TreeList;
  typedef std::list<IupLeafWrapper*> LeafList;

  TreeList _subTrees;
  LeafList _leaves;

  IupTreeWrapper* _parent;

  int tree(void);

  int subTreeFrontPosition(void);
  int subTreeBackPosition(void);

  int leafFrontPosition(void);
  int leafBackPosition(void);

  Ihandle* _tree;

  char* _name;
  void* _data;

  int getDepth(void* userId);

  void setTree(Ihandle* tree) { _tree = tree; }
};


#endif
